
# This file was generated by 'versioneer.py' (0.21) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "date": "2021-12-16T17:07:57+0100",
 "dirty": false,
 "error": null,
 "full-revisionid": "d4096a016b10d67fb5fd06308d1b7bc530f8f802",
 "version": "1.0b1"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)



from .drb_impl_signature import DrbZarrSignature
from .zarr_data_set_node import DrbZarrArrayNode, DrbZarrGroupNode
from .zarr_node_factory import DrbZarrFactory, DrbZarrNode
from . import _version

__version__ = _version.get_versions()['version']

del _version

__all__ = [
    'DrbZarrFactory',
    'DrbZarrNode',
    'DrbZarrNode',
    'DrbZarrArrayNode',
    'DrbZarrGroupNode',
    'DrbZarrSignature'
]



import os
import unittest
from pathlib import Path
import io

from drb.exceptions import DrbException
from drb_impl_file import DrbFileFactory, DrbFileAttributeNames

from drb_impl_zarr import DrbZarrNode, DrbZarrFactory


class TestDrbNodeFactoryZarr(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))

    zarr_fake = current_path / "files" / "fake.zarr"

    zarr_ok2 = current_path / "files" / "sample.zarr"
    zarr_without_group = current_path / "files" / "example.zarr"
    empty_file = current_path / "files" / "empty.files"

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbZarrFactory().create(self.node_file)
        return self.node

    def test_opened_file_node(self):
        node = self.open_node(str(self.zarr_ok2))

        self.assertIsInstance(node, DrbZarrNode)
        self.assertEqual(node.name, self.node_file.name)
        self.assertEqual(node.namespace_uri, self.node_file.namespace_uri)

    def test_base_node(self):
        node = self.open_node(str(self.zarr_ok2))

        self.assertEqual(node.parent, self.node_file.parent)
        self.assertEqual(node.value, self.node_file.value)

        self.assertIsInstance(node, DrbZarrNode)

        self.assertEqual(len(node), 1)
        self.assertTrue(node.has_child())

    def test_base_node_get_child(self):
        node = self.open_node(str(self.zarr_ok2))

        self.assertEqual(node[0].name, '.')
        self.assertEqual(len(node.children), 1)

    def test_base_node_attribute(self):
        node = self.open_node(str(self.zarr_ok2))

        self.assertEqual(node.attributes, self.node_file.attributes)

        self.assertEqual(node.get_attribute(DrbFileAttributeNames.
                                            DIRECTORY.value),
                         self.node_file.get_attribute(DrbFileAttributeNames.
                                                      DIRECTORY.value))

    def test_base_node_impl(self):
        node = self.open_node(str(self.zarr_ok2))

        impl_base_file = io.BufferedIOBase

        # because it is directory no impl
        self.assertFalse(node.has_impl(impl_base_file))

        with self.assertRaises(Exception):
            node.get_impl(impl_base_file)

    def test_path(self):
        node = self.open_node(str(self.zarr_ok2))

        root_node = node[0]

        self.assertEqual(root_node.path.path, node.path.path + '/.')

    def test_fake(self):
        node = self.open_node(str(self.zarr_fake))

        self.assertEqual(node.name, "fake.zarr")

        with self.assertRaises(DrbException):
            len(node)



import os
import uuid

from drb.factory import DrbSignature, DrbSignatureType, DrbFactory

from drb import DrbNode
from drb_impl_zip import DrbZipNode

from .zarr_node_factory import DrbZarrFactory


class DrbZarrSignature(DrbSignature):
    def __init__(self):
        self._factory = DrbZarrFactory()

    @property
    def uuid(self) -> uuid.UUID:
        return uuid.UUID('56e6509c-3666-11ec-8d3d-0242ac130003')

    @property
    def label(self) -> str:
        return 'zarr'

    @property
    def category(self) -> DrbSignatureType:
        return DrbSignatureType.FORMATTING

    @property
    def factory(self) -> DrbFactory:
        return self._factory

    def match(self, node: DrbNode) -> bool:
        supported_ext = {
            '.zarr'
        }

        if isinstance(node, DrbZipNode):
            try:
                if node.has_child() and node['.zarray'] is not None:
                    return True
            except Exception:
                pass

        filename, file_extension = os.path.splitext(node.path.filename)
        return file_extension and file_extension.lower() in supported_ext



import enum
from abc import ABC
from typing import Any, List, Optional, Union, Dict, Tuple

import drb
import zarr

from drb import DrbNode, AbstractNode
from drb.exceptions import DrbNotImplementationException, DrbException
from drb.path import ParsedPath


class DrbZarrAttributeNames(enum.Enum):
    READ_ONLY = 'read_only'


class DrbZarrDataSetNode(AbstractNode, ABC):

    def __init__(self, parent: DrbNode, name: str,
                 data_set: Union[zarr.hierarchy.Group, zarr.core.Array]):
        super().__init__()

        self._data_set = data_set
        self._parent: DrbNode = parent
        self._name = name
        self._path = None
        self._attributes: Dict[Tuple[str, str], Any] = None

    @property
    def namespace_uri(self) -> Optional[str]:
        return None

    def close(self) -> None:
        pass

    @property
    def path(self) -> ParsedPath:
        if self._path is None:
            self._path = self.parent.path / self.name
        return self._path

    @property
    def parent(self) -> Optional[DrbNode]:
        return self._parent

    @property
    def name(self) -> str:
        return self._name

    @property
    def attributes(self) -> Dict[Tuple[str, str], Any]:
        if self._attributes is None:
            self._attributes = {}
            for key in self._data_set.attrs.keys():
                self._attributes[key, None] = self._data_set.attrs[key]
            self._attributes[DrbZarrAttributeNames.READ_ONLY.value, None] = \
                self._data_set.read_only
        return self._attributes

    def get_attribute(self, name: str, namespace_uri: str = None) -> Any:
        key = (name, namespace_uri)
        if key in self.attributes.keys():
            return self.attributes[key]
        raise DrbException(f'Attribute not found name: {name}, '
                           f'namespace: {namespace_uri}')

    @property
    def value(self) -> Optional[Any]:
        return None

    def has_impl(self, impl: type) -> bool:
        if impl is self._data_set.__class__:
            return True

    def get_impl(self, impl: type) -> Any:
        if self.has_impl(impl):
            return self._data_set
        raise DrbNotImplementationException(f'no {impl} '
                                            f'implementation found')

    @staticmethod
    def create_node_from_data_set(parent, data_set):
        name = data_set.name
        if name and name[0] == '/':
            name = name[1:]
        if not name:
            name = '.'
        if isinstance(data_set, zarr.hierarchy.Group):
            node = DrbZarrGroupNode(parent, name, data_set)
        else:
            node = DrbZarrArrayNode(parent, name, data_set)
        return node


class DrbZarrGroupNode(DrbZarrDataSetNode):

    def __init__(self, parent: DrbNode, name: str,
                 data_set: Union[zarr.hierarchy.Group, zarr.core.Array]):
        super().__init__(parent, name, data_set)
        self._children: List[DrbNode] = None

    @property
    @drb.resolve_children
    def children(self) -> List[DrbNode]:
        if self._children is None:
            self._children = []
            self._data_set.visitvalues(self.add_data_set_children)
        return self._children

    def has_child(self) -> bool:
        return len(self.children) > 0

    def add_data_set_children(self, data_set):
        child = DrbZarrDataSetNode.create_node_from_data_set(self, data_set)
        self._children.append(child)

    def _get_named_child(self, name: str, namespace_uri: str = None,
                         occurrence: int = 1) -> Union[DrbNode, List[DrbNode]]:
        if occurrence is None or (not isinstance(occurrence, int)
                                  or occurrence < 0):
            raise DrbException(f'Invalid occurrence: {occurrence}')
        if self._children is None:
            try:
                if namespace_uri is None:
                    data_set = self._data_set[name]
                    if occurrence == 0:
                        return [self.create_node_from_data_set(self, data_set)]
                    elif occurrence == 1:
                        return self.create_node_from_data_set(self, data_set)
                raise DrbException(f'Child ({name},{occurrence}) not found')
            except (IndexError, TypeError, KeyError) as error:
                raise DrbException(f'Child ({name},{occurrence}) not found') \
                    from error
        else:
            return super()._get_named_child(name, namespace_uri, occurrence)


class DrbZarrArrayNode(DrbZarrDataSetNode):

    def __init__(self, parent: DrbNode, name: str,
                 data_set: Union[zarr.hierarchy.Group, zarr.core.Array]):
        super().__init__(parent, name, data_set)

    @property
    def children(self) -> List[DrbNode]:
        return []

    def has_child(self) -> bool:
        return False



import os
import sys
import unittest
from pathlib import Path

from drb.factory import DrbFactoryResolver
from drb.utils.logical_node import DrbLogicalNode

from drb.exceptions import DrbFactoryException


class TestDrbZarrSignature(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))

    zarr_ok2 = current_path / "files" / "sample.zarr"
    not_zarr_files = current_path / "files" / "empty.file"

    @classmethod
    def setUpClass(cls) -> None:
        path = os.path.dirname(__file__)

        cls.mock_package_path = os.path.abspath(
            os.path.join(path, 'resources'))
        sys.path.append(cls.mock_package_path)

        cls.resolver = DrbFactoryResolver()

    @classmethod
    def tearDownClass(cls) -> None:
        sys.path.remove(cls.mock_package_path)

    def test_resolve_ok(self):
        node = DrbLogicalNode(self.zarr_ok2)
        signature, node = self.resolver.resolve(node)
        self.assertEqual(signature.label, 'zarr')

    def test_resolve_no_zarr(self):
        node = DrbLogicalNode(self.not_zarr_files)
        try:
            signature, node = self.resolver.resolve(node)
            self.assertNotEqual(signature.label, 'zarr')
        except DrbFactoryException:
            pass



from typing import Any, List, Dict, Tuple, Optional

from drb import DrbNode, AbstractNode
from drb.factory import DrbFactory
from drb.path import ParsedPath
import zarr

from drb_impl_zarr.exceptions import DrbZarrNodeException
from drb_impl_zarr.zarr_data_set_node import DrbZarrDataSetNode


class DrbZarrNode(AbstractNode):

    def __init__(self, base_node: DrbNode):
        super().__init__()

        self.base_node = base_node
        self._children: List[DrbNode] = None

    @property
    def parent(self) -> Optional[DrbNode]:
        return self.base_node.parent

    @property
    def path(self) -> ParsedPath:
        return self.base_node.path

    @property
    def name(self) -> str:
        return self.base_node.name

    @property
    def namespace_uri(self) -> Optional[str]:
        return self.base_node.namespace_uri

    @property
    def value(self) -> Optional[Any]:
        return self.base_node.value

    @property
    def attributes(self) -> Dict[Tuple[str, str], Any]:
        return self.base_node.attributes

    def get_attribute(self, name: str, namespace_uri: str = None) -> Any:
        return self.base_node.get_attribute(name, namespace_uri)

    @property
    def children(self) -> List[DrbNode]:
        if self._children is None:
            try:
                root_data_set = zarr.open(self.base_node.path.name)
                root_node = DrbZarrDataSetNode.\
                    create_node_from_data_set(self, root_data_set)
                self._children = [root_node]
            except Exception as e:
                raise DrbZarrNodeException(f'Unable to read zarr file'
                                           f' {self.name} ') from e
        return self._children

    def has_child(self) -> bool:
        return True

    def has_impl(self, impl: type) -> bool:
        return self.base_node.has_impl(impl)

    def get_impl(self, impl: type) -> Any:
        return self.base_node.get_impl(impl)

    def close(self):
        self.base_node.close()


class DrbZarrFactory(DrbFactory):

    def _create(self, node: DrbNode) -> DrbNode:
        return DrbZarrNode(base_node=node)



import io
import os
import unittest
from pathlib import Path

import zarr.hierarchy
from drb.exceptions import DrbNotImplementationException, DrbException
from drb_impl_file import DrbFileFactory

from drb_impl_zarr import DrbZarrFactory
from drb_impl_zarr.zarr_data_set_node import DrbZarrGroupNode, \
    DrbZarrArrayNode, DrbZarrAttributeNames


class TestDrbGroupNodeZarr(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))

    zarr_group = current_path / "files" / "group.zarr"

    zarr_ok2 = current_path / "files" / "sample.zarr"
    zarr_without_group = current_path / "files" / "example.zarr"

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbZarrFactory().create(self.node_file)
        return self.node

    def test_first_group(self):
        node = self.open_node(str(self.zarr_ok2))

        root_node = node[0]

        self.assertIsInstance(root_node, DrbZarrGroupNode)
        self.assertEqual(root_node.name, '.')
        self.assertEqual(root_node.namespace_uri, None)

    def test_children_group(self):
        node = self.open_node(str(self.zarr_ok2))

        root_node = node[0]

        self.assertIsInstance(root_node, DrbZarrGroupNode)
        self.assertTrue(root_node.has_child())
        self.assertEqual(len(root_node), 4)

    def test_get_child_group(self):
        node = self.open_node(str(self.zarr_ok2))

        root_node = node[0]

        self.assertIsNotNone(root_node[('traffic_light_faces', 1)])
        self.assertIsNotNone(root_node['traffic_light_faces'])

        child_t_l_faces = root_node['traffic_light_faces']
        self.assertEqual(child_t_l_faces.name, 'traffic_light_faces')

        with self.assertRaises(KeyError):
            root_node[('traffic_light_faces', 2)]

        with self.assertRaises(KeyError):
            root_node[('child_not_exist', 1)]

        root_node.close()

    def test_group_attributes(self):
        node = self.open_node(str(self.zarr_ok2))

        root_node = node[0]

        attr = root_node.get_attribute(DrbZarrAttributeNames.READ_ONLY.value)
        self.assertIsInstance(attr, bool)

    def test_group_attributes_fails(self):
        node = self.open_node(str(self.zarr_ok2))

        root_node = node[0]
        with self.assertRaises(DrbException):
            root_node.get_attribute('empty_attr')

    def test_group_parent(self):
        node = self.open_node(str(self.zarr_ok2))

        root_node = node[0]

        self.assertEqual(root_node.parent, node)
        level1 = root_node[0]

        self.assertEqual(level1.parent, root_node)

    def test_group_value(self):
        node = self.open_node(str(self.zarr_ok2))

        root_node = node[0]

        self.assertEqual(root_node.value, None)

    def test_zarr_without_group(self):
        node = self.open_node(str(self.zarr_without_group))

        root_node = node[0]
        self.assertTrue(isinstance(root_node, DrbZarrArrayNode))

        self.assertFalse(root_node.has_child())

    def test_group_get_children_at(self):
        node = self.open_node(str(self.zarr_ok2))

        root_node = node[0]

        level_node_1 = root_node[0]
        self.assertIsNotNone(level_node_1)
        self.assertEqual(level_node_1.name, "agents")

        with self.assertRaises(IndexError):
            root_node[5]
        self.assertEqual(root_node[-2], root_node[len(root_node) - 2])

    def test_group_get_named_children(self):
        node = self.open_node(str(self.zarr_ok2))

        root_node = node[0]

        child3 = root_node[2]
        self.assertIsNotNone(child3)
        self.assertEqual(child3.name, 'scenes')

        child_by_name = root_node[child3.name]

        self.assertEqual(child3, child_by_name)

    def test_group_impl(self):
        node = self.open_node(str(self.zarr_ok2))

        root_node = node[0]

        self.assertTrue(root_node.has_impl(zarr.hierarchy.Group))

        impl = root_node.get_impl(zarr.hierarchy.Group)

        self.assertIsInstance(impl, zarr.hierarchy.Group)

    def test_dimension_impl_not_supported(self):
        node = self.open_node(str(self.zarr_ok2))

        root_node = node[0]

        self.assertFalse(root_node.has_impl(io.BufferedIOBase))

        with self.assertRaises(DrbNotImplementationException):
            root_node.get_impl(io.BufferedIOBase)



import versioneer
from setuptools import find_packages, setup

with open('requirements.txt') as f:
    REQUIREMENTS = f.readlines()

with open('README.md') as fh:
    long_description = fh.read()

setup(
    name='drb-impl-zarr',
    packages=find_packages(include=['drb_impl_zarr']),
    description='DRB Zarr implementation',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='GAEL Systems',
    author_email='drb-python@gael.fr',
    url='https://gitlab.com/drb-python/impl/zarr',
    install_requires=REQUIREMENTS,
    setup_requires=['setuptools_scm'],
    test_suite='tests',
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent"
    ],
    python_requires='>=3.8',
    entry_points={'drb.impl': 'zarr = drb_impl_zarr.drb_impl_signature'},
    use_scm_version=True,
    version=versioneer.get_version(),
    data_files=[('.', ['requirements.txt'])],
    cmdclass=versioneer.get_cmdclass(),
    include_package_data=True,
)



import os
import unittest
from pathlib import Path

import zarr.hierarchy
from drb.exceptions import DrbNotImplementationException, DrbException
from drb_impl_file import DrbFileFactory

from drb_impl_zarr import DrbZarrFactory
from drb_impl_zarr.zarr_data_set_node import DrbZarrArrayNode


class TestDrbGroupNodeZarr(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))

    zarr_ok2 = current_path / "files" / "sample.zarr"
    zarr_without_group = current_path / "files" / "example.zarr"

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbZarrFactory().create(self.node_file)
        return self.node

    def test_array_parent(self):
        node = self.open_node(str(self.zarr_ok2))

        root_node = node[0]
        second_child = root_node[1]

        self.assertEqual(second_child.parent, root_node)

    def test_array_has_no_child(self):
        node = self.open_node(str(self.zarr_without_group))

        root_node = node[0]
        self.assertTrue(isinstance(root_node, DrbZarrArrayNode))

        self.assertFalse(root_node.has_child())
        self.assertEqual(len(root_node), 0)

    def test_array_get_children_at(self):
        node = self.open_node(str(self.zarr_ok2))

        root_node = node[0]
        second_child = root_node[1]

        with self.assertRaises(IndexError):
            second_child[0]

    def test_array_impl(self):
        node = self.open_node(str(self.zarr_without_group))

        root_node = node[0]

        self.assertTrue(root_node.has_impl(zarr.core.Array))

        impl = root_node.get_impl(zarr.core.Array)

        self.assertIsInstance(impl, zarr.core.Array)

    def test_dimension_impl_not_supported(self):
        node = self.open_node(str(self.zarr_without_group))

        root_node = node[0]

        self.assertFalse(root_node.has_impl(zarr.hierarchy.Group))

        with self.assertRaises(DrbNotImplementationException):
            root_node.get_impl(zarr.hierarchy.Group)

    def test_array_attributes(self):
        node = self.open_node(str(self.zarr_without_group))

        root_node = node[0]

        attr = root_node.get_attribute('myattr')
        self.assertEqual(attr, 'myattr_value')

    def test_array_attributes_fails(self):
        node = self.open_node(str(self.zarr_without_group))

        root_node = node[0]
        with self.assertRaises(DrbException):
            root_node.get_attribute('empty_attr')
